function [xx1,xx2]=creategrid(n1,n2,gfunc,doplot);
  % 
  % Create a curvilinear grid to use for evaluation and plotting.
  % gamma1 is the top boundary and gamma2 is the bottom boundary.
  %  
  % n1 and n2 are the number of points in each direction.
  %
  % The range of x2 is supposed to be [0,1]
  %  
  h2=1/(n2-1);
  x2=0:h2:1;
  
  g1 = gamma1(x2,gfunc);
  g2 = gamma2(x2,gfunc);
  z1 = g2-g1;

  h1 = 1/(n1-1);
  x1 = 0:h1:1;

  [xx2,xx1]=meshgrid(x2,x1);
  %
  % Scale the vertical coordinate to follow the boundary curves.
  %
  for k=1:length(z1)
    xx1(:,k) = g1(k) + z1(k)*xx1(:,k);
  end  
    
  if (nargin==3 | (nargin==4 & doplot>0))
    figure(ceil(doplot)),clf
    mesh(xx2,xx1,zeros(size(xx1)))
    view(2)
    hold on
    axis ij
    xg = 0:0.01:1;
    plot(xg,gamma1(xg,gfunc),'r--')
    plot(xg,gamma2(xg,gfunc),'r--')
    plot([0 0],[gamma1(0,gfunc) gamma2(0,gfunc)],'r--')
    plot([1 1],[gamma1(1,gfunc) gamma2(1,gfunc)],'r--')
    title('The evaluation points')
  end
 