function [xl,xi,xr,xtb]=createpoints(n1,n2,rproc,gfunc,doplot);
  % 
  % Create randomly uniform points in a domain with gamma1 as the 
  % top boundary and gamma2 as the bottom boundary.
  %  
  % n1 and n2 are used for determining the average spacing between points
  % in the two directions. The left boundary is used for initiating h1.
  %
  % The range of x2 is supposed to be [0,1]
  %  
  h2=1/(n2-1);
  x2=0:h2:1;

  g1 = gamma1(x2,gfunc);
  g2 = gamma2(x2,gfunc);
  z1 = g2-g1;
  h1 = z1(1)/(n1-1);
  %
  % Compute the appropiate number of points for each vertical line
  %
  nz = ceil(z1/h1) + 1;
  nz = max(nz,3);
  hz = z1./(nz-1);
  %
  % Points at the left boundary
  %
  xl(:,1) = (g1(1)+hz(1):hz(1):g2(1)-hz(1))';
  xl(:,2) = x2(1)*ones(nz(1)-2,1); 
  %
  % Points at the right boundary
  %
  xr(:,1) = (g1(end)+hz(end):hz(end):g2(end)-hz(end))';
  xr(:,2) = x2(end)*ones(nz(end)-2,1);
  %
  % Interior points
  %
  xi = zeros(0,2);
  for k=2:n2-1
    xi = [xi; (g1(k)+hz(k):hz(k):g2(k)-hz(k))' x2(k)*ones(nz(k)-2,1)];
  end             
  %
  % Add a random disturbance to all points. What if anisotropic?
  % 
  %d = diff(xl(:,1));
  %hi = [d(1); min(abs(d(1:end-1)), abs(d(2:end))); d(end)]
  hi = xl(2,1)-xl(1,1);
  xl(:,1) = xl(:,1) + hi*rproc*2.*(rand(size(xl,1),1)-0.5);
  
  %d = diff(xr(:,1));
  %hi = [d(1); min(abs(d(1:end-1)), abs(d(2:end))); d(end)]
  hi = xr(2,1)-xr(1,1);
  xr(:,1) = xr(:,1) + hi*rproc*2.*(rand(size(xr,1),1)-0.5);
  
  d = diff(xi(:,1));
  hi = [d(1); min(abs(d(1:end-1)), abs(d(2:end))); d(end)];
  xi(:,1) = xi(:,1) + hi*rproc*2.*(rand(size(xi,1),1)-0.5);
  %
  % This does not work, becasue the nodes are organized in the other direction
  %
  %d = diff(xi(:,2));
  %hi = [d(1); min(abs(d(1:end-1)), abs(d(2:end))); d(end)]
  hi=h2;
  xi(:,2) = xi(:,2) + hi*rproc*2.*(rand(size(xi,1),1)-0.5);
  %
  % The points on the top and bottom boundaries cannot be pushed around
  % arbitrarily. Try instead to get an even spacing. First and last point
  % stay at the corners.
  %  
  test(:,2) = (0:1/(10*n2):1)';
  test(:,1) = gamma1(test(:,2),gfunc);
  ht2 = diff(x2);
  curr = 1;
  xtb(1,:) = test(1,:); 
  for g=1:2
    for k=2:length(test(:,1)-1)
      dist = norm(test(k,:)-xtb(curr,:));
      [~,pos] = min(abs(test(k,2)-x2));
      loch = ht2(min(pos,end));
      %
      % If the test point is sufficiently far away, we pick it.
      %
      nexth = loch*(1+rproc*2*(rand(1)-0.5));
      if (dist >= nexth-1e-6) % To make sure that i don't miss the
                              % uniform case.
        xtb(curr+1,:) = test(k,:);
        curr = curr + 1;
      end
    end
    %
    % If we are too close to the final point
    %
    if (norm(test(end,:)-xtb(curr,:)) < loch*rproc*0.7)
      curr = curr-1;
    end  
    xtb(curr+1,:) = test(end,:);
    curr = curr + 1;
    if (g==1)
      %
      % Initialize the next boundary
      %
      test(:,1) = gamma2(test(:,2),gfunc);
      xtb(curr+1,:) = test(1,:);
      curr = curr + 1;
    end  
  end  
    
  if (nargin==5 | (nargin==6 & doplot>0))
    xy = [xl;xi;xr;xtb];
    figure(ceil(doplot)),clf
    H=plot(xy(:,2),xy(:,1),'o');
    set(H,'LineWidth',2)
    hold on
    axis ij
    xg = 0:0.01:1;
    G(1)=plot(xg,gamma1(xg,gfunc),'r--');
    G(2)=plot(xg,gamma2(xg,gfunc),'r--');
    G(3)=plot([0 0],[gamma1(0,gfunc) gamma2(0,gfunc)],'r--');
    G(4)=plot([1 1],[gamma1(1,gfunc) gamma2(1,gfunc)],'r--');
    set(G,'LineWidth',2)
    %title('The node points')
    set(gca,'FontSize',14)
    xlabel('x_2')
    ylabel('x_1')
  end
 