%
% Do the error estimate with local alpha and beta? Or scale everything to one
% Maybe I can go mode by mode, and stop where the overall result is
% negligible
%
function errest=errest2(xr1,xr2,res,kappa)
%
% First we just organize the data
%
  res = reshape(res,size(xr1));
  nr = size(xr1,2)/2;
  L2 = xr2(end,end)-xr2(1,end); % Same for all
  L1 = xr1(end,:)-xr1(1,:); % For each column
  
  n1 = size(xr1,1);
  h1 = 1/(n1-1);
  x1=(0:h1:1)';  
  %mu_l = floor(kappa*Ll/pi);
  %mu_r = floor(kappa*Lr/pi);
  %
  % Loop over the modes until the contribution to the error estimate does
  % not change significantly.
  %
  m = 1;
  maxnum = 20; % Assuming that 20 modes are enough
  errest = 0;
  relContrib = 1;
  while relContrib > 1e-2
    %
    % Compute alpha(x2) and beta(x2) for mode m
    %
    alpha = m*pi./L1;
    beta = sqrt(kappa^2-alpha.^2);
    %
    % Creat a generic sine function and later scale with actual step.
    %
    psi_m = sqrt(2)*sin(m*pi*x1);
    %
    % Compute the coefficient of this mode across the domain.
    % Scalar product with Trapezoidal rule with psi_m. The integrand is
    % zero at both end points.
    %
    r_m = (psi_m'*res)*h1.*L1;

    %figure(10)
    %semilogy(xr2(1,:),abs(r_m)), hold on
    %
    % Form the beta-part of the integrand
    %
    fbeta = 1/sqrt(2)./beta;
    pos = find(real(beta)==0);
    fbeta(pos) = sqrt(2)./abs(beta(pos)).^2.*(1-exp(-0.5*abs(beta(pos))));
    %
    % Integration by the trapezoid rule in the other direction.
    % even if we have only the result at the boundaries, we integrate as
    % if we have a h2-size step in the middle.
    %
    int = fbeta.*abs(r_m);
    %semilogy(xr2(1,:),int)
    %figure(11)
    %semilogy(fbeta), hold on    

    h2 = xr2(1,2)-xr2(1,1);
    errContrib = 0.5*h2*sum(int(1:end-1)+int(2:end));
    errest = errest + errContrib;
    relContrib = errContrib./errest;
    m = m+1;
    if (m>maxnum)
      warning('Exiting the erorr estimate after 20 modes without convergence')
      break
    end  
  end