clear all
close all

doplot = 1;   % For graphics to be displayed
gmode = 1;    % Grid mode:  0 = load file, 1 = create new
mode = 0;     % Problem mode: 0 = let the problem decide the number of
              % propagating modes, >0 = use only this mode
smode=2;      % Ref solution mode: 0 = load reference solution, 1 =  use
              % analytic, <0 or >1 no reference solution
qr = 0;       % 1 if RBF-QR should be used, 0 otherwise
rproc = 0.25; % How much in terms of the step sizes to jitter the points
gfunc = 1;    % Duct shape: 0-4 give different choices
phi = 'mq';   % The RBF to use 'mq' = multiquadric, 'gs' = Gaussian
tol=1e-14;    % The tolerance for the qudrature
xs = 0.3;     % The vertical coordinate of the sound source
W = 3;      % The number of wavelengths
D1 = 1;     % The depth at the left boundary (depends on gfunc)
D2 = 1;       % The depth at the right boundary (depends on gfunc)
n1 = D1*W*12;  % The number of points vertically
n2 = D2*W*12;  % The number of points horizontally

h=max(D1/(n1-1),D2/(n2-1));

Cep = 1.5;
ep = Cep/sqrt(h); % The shape parameter

kappa = 2*pi*W; % The wave number

if (gmode==0)
  fname = input('Give the file name > ','s');
  load(fname);
else
  [xl,xi,xr,xtb]=createpoints(n1,n2,rproc,gfunc,doplot);
end
  
xall = [xl;xi;xr;xtb];
N = size(xall,1)
%
% Use diffmat to initialize RBF-QR, but then we use the raw calls.
%
if (qr)
  [~,Psi]=RBF_QR_diffmat_2D('1',xall,xall,ep);
else
  Psi=[];
end
%
% Allocate and form the RBF approximation matrix
%
[A,fcav] = form_HZ_A(phi,ep,gfunc,tol,kappa,xl,xi,xr,xtb,mode,qr,Psi);
%cA=cond(A)
[f,Am]=form_HZ_rhs(gfunc,kappa,xs,size(A,1),xl,mode);
%
% Solve for the coefficients
%
lambda = A\f;
%
% Construct a grid for evaluating the solution
%
n1e=32;n2e=316;
[xx1,xx2]=creategrid(n1e,n2e,gfunc,doplot*(doplot+1));
%
% Form the evaluation matrix and remove A to leave room
%
clear A f
B = form_HZ_B(phi,ep,xx1,xx2,xall,qr,Psi);
% 
% Compute and visualize the solution
%
sol_rbf = B*lambda;
if (doplot)
  sol_cont(xx1,xx2,sol_rbf,doplot+2,'The RBF solution');
end
%
% Construct a grid for the residual or use old one. 
%
xr1 = xx1;
xr2 = xx2;
%
% Form the residual evaluation matrix and remove B to make room
%
clear B
C = form_HZ_C(phi,ep,kappa,xr1,xr2,xall,qr,Psi);
res_rbf = C*lambda;
%
% Compute the error estimate or plot the residual
%
maxapp = errest2(xr1,xr2,res_rbf,kappa);
disp(['The error estimate says ' num2str(maxapp)])
relest = maxapp/max(max(abs(sol_rbf)));
disp(['Relative to max solution ' num2str(relest)])
%
% Compute errors if it is at all possible.
%
if (smode==0) 
  fname = input('Give the solution file name > ','s');
  load(fname);
elseif (smode==1)
  if (gfunc~=0 & gfunc~=4 )
    error('No known exact solution')
  else
    sol_ref = zeros(size(xx1));
    g1=0;g2=0.1;
    for k=1:length(Am)
      sol_ref = sol_ref + Am(k)*psifun(xx1(:,1),k,g1,g2)* ...
                exp(i*beta(k,g1,g2,kappa)*xx2(1,:));
    end
  end  
end

if (smode==0 | smode==1)
  error = sol_rbf-sol_ref(:);
  sol_cont(xx1,xx2,abs(error),doplot+4,'The error');
  sol_cont(xx1,xx2,sol_ref(:),doplot+5,'The reference solution');

  maxerr = max(max(abs(error)));
  maxsol = max(max(abs(sol_ref)));
  disp(['The maximum error wrt reference is ' num2str(maxerr)])
  disp(['The maximum of the solution is ' num2str(maxsol)])
  %
  % Note that this is assuming that we have large errors at large
  % solution values.
  %
  disp(['leading to a relative error of about ' num2str(maxerr/maxsol)])
end


