%
% To solve the 1D-problem for very small epsilon, we need RBF-QR, but for
% larger epsilon, we prefer to use MQ. Implement both cases with a switch
% for the size of epsilon. Make sure that we get the same results as before.
% Interior eqs -phi''-kappa^2phi
% bc 1 : -phi'-ikappa phi rhs -2ikappa
% bc 2 : phi'-ikappa phi
function [xe,ue,uex,err]=solver(phi,ep,kappa,N,Ne)
  
  xc = linspace(0,1,N)';
  xe = linspace(0,1,Ne)';
  
  r = xcdist(xc,xc,1);
  re = xcdist(xe,xc,1);
 
  if (phi~='gs' | (phi=='gs' & ep>2))
    % Phi
    A0 = RBFmat(phi,ep,r,'0');
    A = A0;
    A([1 end],:) = -i*kappa*A([1 end],:); 
    A(2:end-1,:) = -kappa^2*A(2:end-1,:);
    
    % Phi''
    C = zeros(N);
    C(2:N-1,:) = -1*RBFmat(phi,ep,r(2:N-1,:,:),'L');
  
    % Phi'
    B = zeros(N);
    B([1 end],:) = RBFmat(phi,ep,r([1 end],:,:),'1',1);
    B(1,:) = -B(1,:);
  
    L = A+B+C;
    
    Ae = RBFmat(phi,ep,re,'0');
  else
    %
    % Use RBF-QR to compute the matrices
    %
    [A0,Psi]=RBF_QR_diffmat_1D('0',xc,xc,ep); % First time 
    % Phi
    A = A0;
    A([1 end],:) = -i*kappa*A([1 end],:); 
    A(2:end-1,:) = -kappa^2*A(2:end-1,:);
    
    % Phi''
    C = zeros(N);
    C(2:N-1,:) = -1*RBF_QR_diffmat_1D('xx',xc(2:end-1),Psi);
  
    % Phi'
    B = zeros(N);
    B([1 end],:) = RBF_QR_diffmat_1D('x',xc([1 end]),Psi);
    B(1,:) = -B(1,:);
  
    L = A+B+C;
    
    Ae = RBF_QR_diffmat_1D('0',xe,Psi);
  end
  %
  % The right hand side is zero except at the left boundary
  %
  rhs = zeros(N,1);
  rhs(1) = -2*i*kappa;
  %
  % Solve the system L*lambda = rhs, u=A0*lambda
  %
  LL=L/A0;
  %
  % This is only u at the node points
  %
  u = LL\rhs;
  %
  % Evaluate also at xe 
  %
  ue = (Ae/A0)*u;
  %
  % Exact solution
  %
  uex = exp(i*kappa*xe);
  %
  % Error
  %
  err = ue-uex;
  %figure(1),clf
  %plot(xe,real(ue),xe,real(uex))
  
  %figure(2),clf
  %plot(xe,abs(err))